package code.model

import java.sql.Date
import scala.slick.driver.H2Driver.simple._
import scala.slick.collection.heterogenous._
import syntax._

class RoyaltyDatabase {
  // Map database "y" and "n" values to a proper boolean type
  val YesNoMapper = MappedColumnType.base[Boolean, String](if(_) "y" else "n", _=="y")
 
  case class Administrator(
      id: Int,
      name: String,
      street1: Option[String],
      street2: Option[String],
      city: Option[String],
      state: Option[String],
      zip: Option[String],
      phone: Option[String],
      email: Option[String]
  )
  
  class Administrators(tag: Tag) extends Table[Administrator](tag, "administrator") {
    def id = column[Int]("id", O.PrimaryKey, O.AutoInc, O.DBType("int(10) unsigned"))
    def name = column[String]("name", O.DBType("VARCHAR(255)")) // UNIQUE KEY
    def street1 = column[Option[String]]("street1", O.DBType("VARCHAR(255)"))
    def street2 = column[Option[String]]("street2", O.DBType("VARCHAR(255)"))
    def city = column[Option[String]]("city", O.DBType("VARCHAR(127)"))
    def state = column[Option[String]]("state", O.DBType("VARCHAR(5)"))
    def zip = column[Option[String]]("zip", O.DBType("VARCHAR(10)"))
    def phone = column[Option[String]]("phone", O.DBType("VARCHAR(16)"))
    def email = column[Option[String]]("email", O.DBType("VARCHAR(127)"))
    def * = (id, name, street1, street2, city, state, zip, phone, email) <> (Administrator.tupled, Administrator.unapply)
  }
  val administrators = TableQuery[Administrators]


  class ThirtyThreeCols(tag: Tag) extends Table[
      Int :: Option[Int] :: Option[String] :: String :: Option[Date] ::
      Option[Int] :: Int :: Boolean :: Boolean :: Option[Double] ::
      Boolean :: Option[BigDecimal] :: Option[Double] :: Option[Double] :: Boolean ::
      Option[Double] :: Option[Double] :: Boolean :: Option[Double] :: Boolean ::
      Option[Int] :: Option[Double] :: Float :: Float :: Float ::
      Boolean :: Boolean :: Option[Short] :: Option[String] :: Option[String] ::
      Boolean :: Boolean :: Option[Double]  :: HNil ](tag, "thirty_three") {
    def col1 = column[Int]("col1", O.PrimaryKey, O.AutoInc, O.DBType("int(11)"))
    def col2 = column[Option[Int]]("col2", O.DBType("int(11)")) // KEY
    def col3 = column[Option[String]]("col3", O.DBType("VARCHAR(255)")) // UNIQUE KEY
    def col4 = column[String]("col4", O.DBType("VARCHAR(255)"), O.Default(""))
    def col5 = column[Option[Date]]("col5", O.DBType("date"))

    def col6 = column[Option[Int]]("col6", O.DBType("int(11)"))
    def col7 = column[Int]("col7", O.DBType("int(11)"), O.Default(1))
    def col8 = column[Boolean]("col8", O.Default(true),O.DBType("enum('y','n')"))(YesNoMapper)
    def col9 = column[Boolean]("col9", O.Default(true),O.DBType("enum('y','n')"))(YesNoMapper) // nullable in schema
    def col10 = column[Option[Double]]("col10", O.DBType("double"))
    
    def col11 = column[Boolean]("col11", O.Default(true),O.DBType("enum('y','n')"))(YesNoMapper)
    def col12 = column[Option[BigDecimal]]("col12", O.DBType("decimal(10, 2)"))
    def col13 = column[Option[Double]]("col13", O.DBType("double"))
    def col14 = column[Option[BigDecimal]]("col14", O.DBType("decimal(10, 2)"))
    def col15 = column[Boolean]("col15", O.Default(false),O.DBType("enum('y','n')"))(YesNoMapper)
    
    def col16 = column[Option[Double]]("col16", O.DBType("double"))
    def col17 = column[Option[Double]]("col17", O.DBType("double"))
    def col18 = column[Boolean]("col18", O.Default(false),O.DBType("enum('y','n')"))(YesNoMapper)
    def col19 = column[Option[Double]]("col19", O.DBType("double"))
    def col20 = column[Boolean]("col20", O.Default(true),O.DBType("enum('y','n')"))(YesNoMapper)
    
    def col21 = column[Option[Int]]("col21")
    def col22 = column[Option[Double]]("col22")
    def col23 = column[Float]("col23", O.Default(0))
    def col24 = column[Float]("col24", O.Default(0))
    def col25 = column[Float]("col25", O.Default(0))
  
    def col26 = column[Boolean]("col26", O.Default(false),O.DBType("enum('y','n')"))(YesNoMapper)
    def col27 = column[Boolean]("col27", O.Default(false))(YesNoMapper)
    def col28 = column[Option[Short]]("col28", O.DBType("int(16) unsigned"))
    def col29 = column[Option[String]]("col29", O.DBType("VARCHAR(10,2)"))
    def col30 = column[Option[String]]("col30", O.DBType("VARCHAR(10,2)"))
    def col31 = column[Boolean]("col31", O.Default(true),O.DBType("enum('y','n')"))(YesNoMapper)
    def col32 = column[Boolean]("col32", O.Default(true),O.DBType("enum('y','n')"))(YesNoMapper)
    def col33 = column[Option[Double]]("col33")
    def * = col1 :: col2 :: col3 :: col4 :: col5 :: col6 :: col7 :: col8 ::
             col9 :: col10 :: col11 :: col12 :: col13 :: col14 ::
             col15 :: col16 :: col17 :: col18 :: col19 :: col20 ::
             col21 :: col22 :: col23 :: col24 :: col25 :: col26 :: 
             col27 :: col28 :: col29 :: col30 :: col31 :: col32 :: col33 :: HNil 
             //<>  (ThirtyThreeCol.construct, ThirtyThreeCol.extract)
  
  }
  val thirty_three = TableQuery[ThirtyThreeCols]


}

